;**************************************************************************
;* FileName	: RS232.INC
;* Date 	: 09.10.2002
;* File version : v1.03
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: functions for for rs232 communication
;*		  ==================================
;*		  RS232_SendByte
;*		  RS232_ReceiveByte
;*		  RS232_SendCS
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;* #define	RS232_TxD	PORTA,0		; out
;* #define	RS232_RxD	PORTA,1		; in
;*
;*	include "..\modules\wait.mac"
;*	include "..\modules\wait.inc"
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* #define DEBUG			; for some debug message
;* #define NO_RS232			; for fast simulation
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  22/08/2000 	Initial Release
;* 1.01	  18/02/2001 	add function RS232_SendCS
;* 1.02   15/06/2002	minor change
;* 1.03   09/10/2002    reformatting source
;*
;**************************************************************************

;***** VARIABLES **********************************************************

	ifdef INC_RS232
		cblock
		  RS232_BitNumber
		  RS232_SendBuffer
		  RS232_RecvBuffer
		  RS232_temp
		endc
	endif


;***** SEND SERIAL DATA ***************************************************
;* Send byte
;* Input : W				; byte
;* Output: -
;* Clock : ?
;**************************************************************************
	ifdef INC_RS232
RS232_SendByte	bcf	RS232_TxD		; start bit
		movwf	RS232_SendBuffer                          
		movlw	RS232_BITS		; x bit           
		movwf	RS232_BitNumber
		nop2				; timeing
		
RS232_sendloop	WAIT	RS232_BITTIME, 9

		bcf	STATUS,C		; 0 -> CARRY           
		rrf	RS232_SendBuffer,f
		skipc		
		bcf	RS232_TxD		; clear           
		skipnc
		bsf	RS232_TxD		; or set TxD

		decfsz	RS232_BitNumber,f
		goto	RS232_sendloop

		
		WAIT	RS232_BITTIME, 4

		bsf	RS232_TxD		; stop bit

		WAIT	(RS232_BITTIME * RS232_STOPBITS), 0

		return
	endif



;***** RECEIVE SERIAL DATA ************************************************
;* Receive byte
;* Input : -				; 
;* Output: received byte -> W		;
;* Clock : ?
;**************************************************************************
	ifdef	INC_RS232
RS232_ReceiveByte
		movlw	RS232_BITS		; x bit
		movwf	RS232_BitNumber

		clrf	RS232_RecvBuffer	; clear recieve buffer
RS232_waitrec	clrwdt
		btfsc	RS232_RxD
		goto	RS232_waitrec		; wait for start bit

		WAIT	(RS232_BITTIME + (RS232_BITTIME/2)), 6	; 1.5 bit

RS232_rloop	bcf	STATUS,C
		rrf	RS232_RecvBuffer,f
		btfsc	RS232_RxD
		bsf	RS232_RecvBuffer,7

		WAIT	RS232_BITTIME, 7

		decfsz	RS232_BitNumber,f	
		goto	RS232_rloop

		movf	RS232_RecvBuffer,w
		return
	endif



;***** RS232_SendCS *******************************************************
;* Send constant string
;* Input : -				; 
;* Output: received byte -> W		;
;* Clock : ?
;**************************************************************************
	ifdef	INC_RS232_SCS
RS232_SendCS	movwf	PCLATH

RS232_sendcs1	call	RS232_sendcs_getc
		testw
		skipnz
		return

		call	RS232_SendByte

		incf	RS232_temp,f
		skipnz
		incf	PCLATH,f
		goto	RS232_sendcs1

RS232_sendcs_getc
		movf	RS232_temp,w
		movwf	PCL		; computed goto!

	endif


;**************************************************************************



